"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = BarcodeScanner;

var _utils = require("./utils");

/**
 * Simple JavaScript utility to listen for barcode scanners emulating keyboard
 */
function BarcodeScanner(options) {
  var fun;
  options = (0, _utils.assign)({
    latency: 50,
    minLength: 3,
    element: document.documentElement,
    endKeys: ["Enter", "ArrowDown", "ArrowRight", "End"],
    validKey: /^[a-zA-Z0-9]$/,
    preventDefault: true,
    stopPropagation: true
  }, options);
  var prevTime = 0;
  var code = "";

  function EventHandler(e) {
    var key = e.key,
        timeStamp = e.timeStamp;
    var timeDiff = timeStamp - prevTime;
    prevTime = timeStamp; // Ignore shift key

    if (key === "Shift") return;
    var isValid = options.validKey.test(key);
    var isEndKey = options.endKeys.includes(key);

    if (timeDiff > options.latency) {
      // Maybe a normal key press or start of barcode
      if (!isEndKey && isValid) {
        code = key;
      } else code = "";
    } else if (isValid) {
      // Still scanning
      code += key;
    } else {
      if (isEndKey) {
        // End of barcode
        if (code.length >= options.minLength) {
          fun(code);
          if (options.preventDefault) e.preventDefault();
          if (options.stopPropagation) e.stopPropagation();
        }
      } // Invalid character, reset


      code = "";
    }
  }

  return {
    on: function on(handler) {
      options.element.removeEventListener("keydown", EventHandler, true);
      fun = handler;
      code = "";
      fun && options.element.addEventListener("keydown", EventHandler, true);
    },
    off: function off() {
      options.element.removeEventListener("keydown", EventHandler, true);
    }
  };
}